package org.ow2.easywsdl.tooling.xsd2xml;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.ow2.easywsdl.schema.api.ComplexType;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfAttribute;
import org.ow2.easywsdl.schema.api.absItf.AbsItfElement;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractAttributeImpl;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractElementImpl;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.FormChoice;

public class XSD2XMLImpl extends XSD2XML {

	private Namespace xsi = Namespace.getNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");

	@Override
	public Element generateElement(AbsItfElement eIn, Object values) throws SchemaException {
		Element res = null;
		if(eIn != null) {
			res = this.generateElement(eIn, values, ((AbstractElementImpl) eIn).getSchema().getElementFormDefault(), 1);
		}
		return res;
	}


	public Element generateElement(AbsItfElement eIn, Object values, FormChoice form, int minOccurs)
	throws SchemaException {
		if(values == null) {
			values = "?";
		}

		// create element
		org.jdom.Element eOut = null;

		if(minOccurs > 0) {
			eOut = new org.jdom.Element(eIn.getQName().getLocalPart());

			if(form == FormChoice.QUALIFIED) {
				eOut.setNamespace(org.jdom.Namespace.getNamespace(eIn.getQName().getPrefix(), eIn
						.getQName().getNamespaceURI()));
			} 


			// if complextype, generate under element
			if (((AbstractElementImpl) eIn).getType() instanceof ComplexType) {
				if ((((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) eIn)
						.getType()).getModel()) != null) {

					List<org.ow2.easywsdl.schema.api.Element> items = null;

					if ((((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) eIn)
							.getType())).getSequence() != null) {
						// if sequence
						items = (((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) eIn)
								.getType())).getSequence().getElements();
					} else if ((((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) eIn)
							.getType())).getAll() != null) {
						// if all
						items = (((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) eIn)
								.getType())).getAll().getElements();
					}

					if (items != null) {
						for (final org.ow2.easywsdl.schema.api.Element item : items) {

							// is element
							if (item.getRef() == null) {
								// is an element
								final AbsItfElement e = item;
								FormChoice elmtFrom = null;
								if(e.getForm() != null) {
									elmtFrom = e.getForm();
								} else {
									elmtFrom = form;
								}
								Element child = (Element) this.generateElement(e,
										nextValue(values), form, e.getMinOccurs());
								if(child != null) {
									eOut.addContent(child.detach());
								}
							} else {
								// is a ref on an element
								final List<AbsItfElement> listE = ((AbstractElementImpl) eIn).getParent().getSchema()
								.findElementsInAllSchema(item.getRef());
								if(listE != null && listE.size() > 0) {
									AbsItfElement e = listE.get(0);

									FormChoice elmtFrom = null;
									if(e.getForm() != null) {
										elmtFrom = e.getForm();
									} else {
										elmtFrom = form;
									}
									
									
									Element child = (Element) this.generateElement(e,
											nextValue(values), form, e.getMinOccurs());
									if(child != null) {
										eOut.addContent(child.detach());
									}
								} else {
									throw new SchemaException("Impossible to find element corresponding to this reference: " + item.getRef());
								}
							}
						}

						// handle attribute
						for (final AbsItfAttribute attr : ((ComplexType) ((AbstractElementImpl) eIn)
								.getType()).getAttributes()) {
							final org.jdom.Attribute a = this.generateAttribute(
									eIn, attr, nextValue(values));
							if(a != null) {

								eOut.setAttribute(a);
							}
						}

					}
				}
			} else {
				// add value on the simpletype
				final String val = nextValue(values);
				if ((val != null)&&(!eIn.isNillable())) {
					eOut.setText(val);
				}
			}

			// add attribute to element
			List<Attribute> attrs = generateAttributes(eIn, values, ((AbstractElementImpl) eIn).getSchema().getAttributeFormDefault());

			// add type attribute on element
			Attribute typeAttr = null;
			if (((AbstractElementImpl) eIn).getType() != null) {
				String type = null;
				// if not a anonymous type
				if (((AbstractElementImpl) eIn).getType().getQName() != null) {
					String prefix = ((AbstractElementImpl) eIn).getParent().getSchema().getAllNamespaces().getPrefix(
							((AbstractElementImpl) eIn).getType().getQName().getNamespaceURI());
					if (prefix != null) {
						type = prefix 
						+ ":"
						+ ((AbstractElementImpl) eIn).getType().getQName().getLocalPart();
						Namespace typeNs = Namespace.getNamespace(prefix, ((AbstractElementImpl) eIn).getParent().getSchema().getAllNamespaces().getNamespaceURI(prefix));
						eOut.addNamespaceDeclaration(typeNs);
						eOut.addNamespaceDeclaration(xsi);
						typeAttr = new Attribute("type", type, xsi);
					} else {
						type = ((AbstractElementImpl) eIn).getType().getQName().getLocalPart();
						eOut.addNamespaceDeclaration(xsi);
						typeAttr = new Attribute("type", type, xsi);
					}
				}
			}

			if(typeAttr != null) {
				attrs.add(typeAttr);
			}

			eOut.setAttributes(attrs);


			Document doc = new Document(eOut); 
		}
		return eOut;
	}

	private List<Attribute> generateAttributes(AbsItfElement eIn, Object values,
			FormChoice attributeFormDefault) {
		List<Attribute> res = new ArrayList<Attribute>();
		if(eIn.getType() instanceof ComplexType) {
			ComplexType ct = (ComplexType) eIn.getType();
			for(org.ow2.easywsdl.schema.api.Attribute attr: ct.getAttributes()) {
				Attribute a = null;
				String prefix = ((AbstractElementImpl) eIn).getSchema().getAllNamespaces().getPrefix(attr.getNamespaceUri());

				if((prefix != null)&&(attr.getNamespaceUri() != eIn.getQName().getNamespaceURI())&&(attributeFormDefault == FormChoice.QUALIFIED)) {
					a = new Attribute(attr.getName(),nextValue(values), Namespace.getNamespace(prefix, attr.getNamespaceUri()));
				} else {
					a = new Attribute(attr.getName(),nextValue(values));
				}
				res.add(a);
			}
		}
		return res;
	}


	@SuppressWarnings("unchecked")
	private String nextValue(final Object values) {
		String res = null;
		if (values instanceof List) {
			res = (String) ((List) values).get(0);
		} else if (values instanceof String) {
			res = (String) values;
		}
		return res;
	}


	@SuppressWarnings("unchecked")
	public org.jdom.Attribute generateAttribute(final AbsItfElement eIn, final AbsItfAttribute aIn, final String value) {
		org.jdom.Attribute attr = null;
		if (aIn.getName() != null) {
			String prefix = null;
			org.jdom.Namespace ns = null;
			if (!aIn.getNamespaceUri().equals(eIn.getQName().getNamespaceURI())) {
				if (((AbstractElementImpl) eIn).getParent().getSchema().getAllNamespaces().getPrefix(
						aIn.getNamespaceUri()) != null) {
					prefix = ((AbstractElementImpl) eIn).getParent().getSchema().getAllNamespaces().getPrefix(
							aIn.getNamespaceUri());
					ns = org.jdom.Namespace.getNamespace(prefix, aIn.getNamespaceUri());
				}
			}
			if(((AbstractAttributeImpl)aIn).getSchema().getAttributeFormDefault() == FormChoice.QUALIFIED) {
				attr = new org.jdom.Attribute(aIn.getName(), value, ns);
			} else {
				attr = new org.jdom.Attribute(aIn.getName(), value, null);
			}
		}
		return attr;
	}





	@Override
	public String printXml(AbsItfElement e, Object value)
	throws SchemaException {
		final org.jdom.Element eOut = this.generateElement(e, value);
		return new XMLOutputter(Format.getPrettyFormat()).outputString(eOut);
	}


	@Override
	public Element addMinOccursEqual0OrArrayElement(AbsItfElement childDefinition,
			Element parentElement) throws SchemaException {

		// verif if child is nillable or an array element
		String maxOccurs = childDefinition.getMaxOccurs();
		int maxOcc = -1;
		if(maxOccurs == null) {
			maxOcc = 1;
		} else if(maxOccurs.toUpperCase().equals("UNBOUNDED")) {
			maxOcc = 10;
		} else {
			maxOcc = Integer.parseInt(maxOccurs);
		}

		Element child = null;
		if(childDefinition.getMinOccurs() == 0 || maxOcc > 1) {

			child = generateElement(childDefinition, "?", ((AbstractElementImpl) childDefinition).getSchema().getElementFormDefault(), 1);

			parentElement.addContent(child.detach());
		} else {
			throw new SchemaException("this element (" + childDefinition.getQName() + ") is not an array or not minOccurs = 0");
		}

		return child;
	}




}
